# curl 169.254.169.254/latest/meta-data/identity-credentials/ec2/security-credentials/
# Do it with V1 and V2, have "latest" be the default but let that value be configurable


def gather(hub):
    ...
